#######################################################################
#                                                                     #
#                                 Pph                                 #
#                                                                     #
#                    The Portable Project Handler                     #
#                                                                     #
#               EPI Pomdapi, INRIA Paris - Rocquencourt               #
#                                                                     #
#  Copyright 2005-2014 INRIA.                                         #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved. This file is distributed only by permission.  #
#                                                                     #
#  Francois Clement <Francois.Clement@inria.fr>                       #
#  Pierre Weis <Pierre.Weis@inria.fr>                                 #
#                                                                     #
#######################################################################

# $Id: configure_usage.profile,v 1.1 2014/02/27 16:36:36 fclement Exp $

#
# Pph system configuration usage.
# Used by user and devel configuration profiles ``configure.profile''.
#

# Installation section.
print_installation_section () {
    INSTALLATION_TITLE="Installation"
    INSTALLATION_BODY=`\
        doc_with_default "--install-root-dir" "-ird" "dir" \
            "installation root directory" \
            "${INSTALL_ROOT_DIR_INTRINSIC_DEFAULT:?}"`
    print_usage_section "$INSTALLATION_TITLE" "$INSTALLATION_BODY"
}

# Compiler section.
check_compiler () {
    test "${PROJECT_C_COMPILER_STATUS:?}" != "None" || \
    test "${PROJECT_CPP_COMPILER_STATUS:?}" != "None" || \
    test "${PROJECT_FORTRAN_COMPILER_STATUS:?}" != "None" || \
    test "${PROJECT_JAVA_COMPILER_STATUS:?}" != "None" || \
    test "${PROJECT_MOCA_COMPILER_STATUS:?}" != "None" || \
    test "${PROJECT_OCAML_BYT_COMPILER_STATUS:?}" != "None" || \
    test "${PROJECT_OCAML_BIN_COMPILER_STATUS:?}" != "None"
}

print_c_compiler_item () {
    if test "${PROJECT_C_COMPILER_STATUS:?}" != "None"; then
        doc "--c-compiler" "-cc" "file" "C compiler"
    fi
}

print_cpp_compiler_item () {
    if test "${PROJECT_CPP_COMPILER_STATUS:?}" != "None"; then
        doc "--cpp-compiler" "-cppc" "file" "C++ compiler"
    fi
}

print_fortran_compiler_item () {
    if test "${PROJECT_FORTRAN_COMPILER_STATUS:?}" != "None"; then
        doc "--fortran-compiler" "-fc" "file" "Fortran compiler"
    fi
}

print_java_compiler_item () {
    if test "${PROJECT_JAVA_COMPILER_STATUS:?}" != "None"; then
        doc "--java-compiler" "-jc" "file" "Java compiler"
    fi
}

print_moca_compiler_item () {
    if test "${PROJECT_MOCA_COMPILER_STATUS:?}" != "None"; then
        doc "--moca-compiler" "-mc" "file" "Moca compiler"
    fi
}

print_ocaml_compiler_item () {
    if test "${PROJECT_OCAML_BYT_COMPILER_STATUS:?}" != "None"; then
        doc "--ocaml-byte-compiler" "-obc" "file" "OCaml byte compiler"
    fi && \
    if test "${PROJECT_OCAML_BIN_COMPILER_STATUS:?}" != "None"; then \
        doc "--ocaml-native-compiler" "-onc" "file" "OCaml native compiler"; \
    fi && \
    if test "$PROJECT_OCAML_BYT_COMPILER_STATUS" != "None" || \
       test "$PROJECT_OCAML_BIN_COMPILER_STATUS" != "None"; then \
        doc "--ocaml-command-search-path" "-ocsp" "path" \
            "OCaml command search path"; \
    fi
}

print_compiler_section () {
    if check_compiler; then
        COMPILER_TITLE="Compilers"
        COMPILER_BODY=`\
            print_c_compiler_item && \
            print_cpp_compiler_item && \
            print_fortran_compiler_item && \
            print_java_compiler_item && \
            print_moca_compiler_item && \
            print_ocaml_compiler_item`
        print_usage_section "$COMPILER_TITLE" "$COMPILER_BODY"
    fi
}

# Preprocessor section.
check_preprocessor () {
    test "${PROJECT_C_PREPROCESSOR_STATUS:?}" != "None" || \
    test "${PROJECT_CPP_PREPROCESSOR_STATUS:?}" != "None" || \
    test "${PROJECT_FORTRAN_PREPROCESSOR_STATUS:?}" != "None" || \
    test "${PROJECT_OCAML_PREPROCESSOR_STATUS:?}" != "None"
}

print_c_preprocessor_item () {
    if test "${PROJECT_C_PREPROCESSOR_STATUS:?}" != "None"; then
        doc "--c-preprocessor" "-cpp" "file" "C preprocessor"
    fi
}

print_cpp_preprocessor_item () {
    if test "${PROJECT_CPP_PREPROCESSOR_STATUS:?}" != "None"; then
        doc "--cpp-preprocessor" "-cpppp" "file" "C++ preprocessor"
    fi
}

print_fortran_preprocessor_item () {
    if test "${PROJECT_FORTRAN_PREPROCESSOR_STATUS:?}" != "None"; then
        doc "--fortran-preprocessor" "-fpp" "file" "Fortran preprocessor"
    fi
}

print_ocaml_preprocessor_item () {
    if test "${PROJECT_OCAML_PREPROCESSOR_STATUS:?}" != "None"; then
        doc "--ocaml-preprocessor" "-opp" "file" "OCaml preprocessor"
    fi
}

print_preprocessor_section () {
    if check_preprocessor; then
        PREPROCESSOR_TITLE="Preprocessors"
        PREPROCESSOR_BODY=`\
            print_c_preprocessor_item && \
            print_cpp_preprocessor_item && \
            print_fortran_preprocessor_item && \
            print_ocaml_preprocessor_item`
        print_usage_section "$PREPROCESSOR_TITLE" "$PREPROCESSOR_BODY"
    fi
}

# Additional options.
print_additional_section () {
    if ${PROJECT_CONFIGURATION_ADDITIONAL_OPTIONS:?}; then
        print_additional_options_documentation
    fi
}

# For developers section.
print_force_item () {
    $ECHO "\
--force, -f	Redo everything, as if \`\`./unconfigure'' was just been run.
		In particular, previous configuration settings are ignored,
		and tarballs are unpacked again."
}

print_devel_section () {
    if ${DEVEL_MODE:?}; then
        DEVEL_TITLE="For developers"
        DEVEL_BODY=`print_force_item`
        print_usage_section "$DEVEL_TITLE" "$DEVEL_BODY"
    fi
}

# Internal use section.
print_internal_use_section () {
    INTERNAL_USE_TITLE="Internal use (undocumented)"
    INTERNAL_USE_BODY="
--private-tool <dir>, -pt <dir>
"
}

#
# Full documentation.
#
usage () {
    print_usage_header
    print_installation_section
    print_compiler_section
    print_preprocessor_section
    print_additional_section
    print_devel_section
    print_service_section
    print_internal_use_section
}
