/***********************************************************************/
/*                                                                     */
/*                            Pio (Fortran)                            */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*               EPI Pomdapi, INRIA Paris - Rocquencourt               */
/*                                                                     */
/*  Copyright 2005-2014 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Francois Clement <Francois.Clement@inria.fr>                       */
/*  Pierre Weis <Pierre.Weis@inria.fr>                                 */
/*  Hend Ben Ameur <hbenameur@yahoo.ca>                                */
/*                                                                     */
/***********************************************************************/

/* $Id: pio_type_constants.h,v 1.10 2014/03/07 19:05:52 fclement Exp $ */

#ifndef _PIO_IO_TYPE_CONSTANTS_H_
#define _PIO_IO_TYPE_CONSTANTS_H_

/* Pio library type classification, implemented as integer constants. */

/* Communication types. */
#define PIO_PHRASE 0
#define PIO_TASK 1
#define PIO_RESULT 2
#define PIO_ERROR 3
#define PIO_SERVICE 4

/* Service names. */
#define PIO_SERVICE_OK 'Ok'
#define PIO_SERVICE_OK_LENGTH 2
#define PIO_SERVICE_KO 'Ko'
#define PIO_SERVICE_KO_LENGTH 2
#define PIO_SERVICE_ALLO 'Allo'
#define PIO_SERVICE_ALLO_LENGTH 4
#define PIO_SERVICE_BYE 'Bye'
#define PIO_SERVICE_BYE_LENGTH 3
#define PIO_SERVICE_START 'Start'
#define PIO_SERVICE_START_LENGTH 5
#define PIO_SERVICE_STOP 'Stop'
#define PIO_SERVICE_STOP_LENGTH 4

/* Value_type = Basic_tag + Tuple_tag + Array_tag. */

/* Basic tags (first tag should be 0). */
#define STRING_TAG 0
#define INT_TAG    1
#define FLOAT_TAG  2
#define LAST_BASIC_TAG  2

/* Tuple tags. */
#define SIMPLE_TAG 10
#define COUPLE_TAG 20
#define TRIPLE_TAG 30
#define FIRST_TUPLE_TAG 10
#define LAST_TUPLE_TAG 30

/* Array tags. */
#define SCALAR_TAG 100
#define VECTOR_TAG 200
#define MULTI_D_TAG 300

#define BASIC_TAG(typ) (mod(typ, 10))
#define TUPLE_TAG(typ) ((mod(typ, 100) / 10) * 10)
#define ARRAY_TAG(typ) (typ / 100)

#define MAKE_COUPLE_TYPE(typ)      (COUPLE_TAG + (typ - SIMPLE_TAG))
#define MAKE_COUPLE_ITEM_TYPE(typ) (SIMPLE_TAG + (typ - COUPLE_TAG))
#define MAKE_TRIPLE_TYPE(typ)      (TRIPLE_TAG + (typ - SIMPLE_TAG))
#define MAKE_TRIPLE_ITEM_TYPE(typ) (SIMPLE_TAG + (typ - TRIPLE_TAG))

#define MAKE_VECTOR_TYPE(typ)      (VECTOR_TAG + (typ - SCALAR_TAG))
#define MAKE_VECTOR_ITEM_TYPE(typ) (SCALAR_TAG + (typ - VECTOR_TAG))

#define MAKE_VECTOR_COUPLE_TYPE(typ) MAKE_VECTOR_TYPE(MAKE_COUPLE_TYPE(typ))
#define MAKE_VECTOR_TRIPLE_TYPE(typ) MAKE_VECTOR_TYPE(MAKE_TRIPLE_TYPE(typ))

#define MAKE_MULTI_D_TYPE(typ)      (MULTI_D_TAG + (typ - SCALAR_TAG))
#define MAKE_MULTI_D_ITEM_TYPE(typ) (SCALAR_TAG + (typ - MULTI_D_TAG))

#define FIRST_SCALAR_SIMPLE_TYPE (SIMPLE_TAG + SCALAR_TAG)
#define LAST_SCALAR_SIMPLE_TYPE  (LAST_BASIC_TAG + FIRST_SCALAR_SIMPLE_TYPE)
#define FIRST_SCALAR_COUPLE_TYPE (MAKE_COUPLE_TYPE(FIRST_SCALAR_SIMPLE_TYPE))
#define LAST_SCALAR_COUPLE_TYPE  (MAKE_COUPLE_TYPE(LAST_SCALAR_SIMPLE_TYPE))
#define FIRST_SCALAR_TRIPLE_TYPE (MAKE_TRIPLE_TYPE(FIRST_SCALAR_SIMPLE_TYPE))
#define LAST_SCALAR_TRIPLE_TYPE  (MAKE_TRIPLE_TYPE(LAST_SCALAR_SIMPLE_TYPE))

#define FIRST_VECTOR_SIMPLE_TYPE (MAKE_VECTOR_TYPE(FIRST_SCALAR_SIMPLE_TYPE))
#define LAST_VECTOR_SIMPLE_TYPE  (MAKE_VECTOR_TYPE(LAST_SCALAR_SIMPLE_TYPE))
#define FIRST_VECTOR_COUPLE_TYPE (MAKE_VECTOR_TYPE(FIRST_SCALAR_COUPLE_TYPE))
#define LAST_VECTOR_COUPLE_TYPE  (MAKE_VECTOR_TYPE(LAST_SCALAR_COUPLE_TYPE))
#define FIRST_VECTOR_TRIPLE_TYPE (MAKE_VECTOR_TYPE(FIRST_SCALAR_TRIPLE_TYPE))
#define LAST_VECTOR_TRIPLE_TYPE  (MAKE_VECTOR_TYPE(LAST_SCALAR_TRIPLE_TYPE))

#define FIRST_MULTI_D_SIMPLE_TYPE (MAKE_MULTI_D_TYPE(FIRST_SCALAR_SIMPLE_TYPE))
#define LAST_MULTI_D_SIMPLE_TYPE  (MAKE_MULTI_D_TYPE(LAST_SCALAR_SIMPLE_TYPE))
#define FIRST_MULTI_D_COUPLE_TYPE (MAKE_MULTI_D_TYPE(FIRST_SCALAR_COUPLE_TYPE))
#define LAST_MULTI_D_COUPLE_TYPE  (MAKE_MULTI_D_TYPE(LAST_SCALAR_COUPLE_TYPE))
#define FIRST_MULTI_D_TRIPLE_TYPE (MAKE_MULTI_D_TYPE(FIRST_SCALAR_TRIPLE_TYPE))
#define LAST_MULTI_D_TRIPLE_TYPE  (MAKE_MULTI_D_TYPE(LAST_SCALAR_TRIPLE_TYPE))

#define MAKE_SCALAR_SIMPLE_TYPE(tag) (tag + FIRST_SCALAR_SIMPLE_TYPE)
#define MAKE_SCALAR_SIMPLE_ITEM_TYPE(typ) (MAKE_SCALAR_SIMPLE_TYPE(BASIC_TAG(typ)))

#define MAKE_SCALAR_ITEM_TYPE(typ) (typ - (ARRAY_TAG(typ) * 100) + SCALAR_TAG)

#define SSTRING MAKE_SCALAR_SIMPLE_TYPE(STRING_TAG)
#define SINT    MAKE_SCALAR_SIMPLE_TYPE(INT_TAG)
#define SFLOAT  MAKE_SCALAR_SIMPLE_TYPE(FLOAT_TAG)

/* Layout_kind type. */
#define C_LAYOUT_KIND 0
#define FORTRAN_LAYOUT_KIND 1
#define PERMUTATION_LAYOUT_KIND 2

#endif
