(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ustring_command_option.ml,v 1.9 2014-01-20 12:04:54 fclement Exp $ *)

module Cmd_opt = Ucommand_option;;

module Make =
  functor (S : Ustring_command_option_spec.STRUCTURE) ->
struct
  open S;;
  
  type t = S.t;;
  
  let print_quoted ppf x = Format.fprintf ppf "%S" (to_string x);;
  
  let { Uaref.get = get; set = set; } = Uaref.make_aref_single name default;;
  let set_from_string = Upervasives.compose set of_string;;
  let print_all ppf = Lib_print.print_list ",@ " print_quoted ppf all;;
  let print_default ppf = print_quoted ppf (get ());;
  
  let buf = Buffer.create 1024;;
  let ppf = Format.formatter_of_buffer buf;;
  Format.fprintf ppf
    "@[<v 8><str>@ \
    @[Set %s to@ <str>.@]@ \
    @[Valid %s are:@ @[<hv 2>%t.@]@]@ \
    Default %s is %t.@]"
    name
    plural_name print_all
    name print_default
  ;;
  let doc = Buffer.contents buf;;
  Buffer.clear buf;;
  
  let option =
    {
     Cmd_opt.key = Ustring.long_key_of_string name;
     spec = Arg.String set_from_string;
     doc = doc;
    }
  ;;
end
;;
